require "fast_crud/version"

module FastCrud
  class FastCrud < ActionController::Base
    def index
      if request.filtered_parameters.has_rkey? /_id/
        @collection = nested_collection
      else
        @collection = model.all
      end
    end

    def show
      @resource = model.find(params[:id])
    end

    def new
      @resource = model.new
    end

    def edit
      @resource = model.find(params[:id])
    end

    def create(location = nil)
      @resource = model.new(model_params)

      if @resource.save
        flash[:success] = "#{model} successfully created!"
        if location
          redirect_to location
        else
          redirect_to @resource
        end
      else
        flash[:danger] = "Error creating #{model}: #{@resource.errors.full_messages.join(' - ')}"
        render 'new'
      end
    end

    def update(location = nil)
      @resource = model.find(params[:id])

      if @resource.update(model_params)
        flash[:success] = "#{model} successfully updated!"
        if location
          redirect_to location
        else
          redirect_to @resource
        end
      else
        flash[:danger] = "Error updating #{model}: #{@resource.errors.full_messages.join(' - ')}"
        render 'edit'
      end
    end

    def destroy(location = nil)
      @resource = model.find(params[:id])
      @resource.destroy
      if location
        redirect_to location
      else
        redirect_to eval("#{controller_name}_path")
      end
    end

    private
    def model
      controller_name.classify.constantize
    end

    def nested_collection
      path = request.fullpath.split("/").reject {|a| a.empty?}.last(3)
      path.first.classify.constantize.find(path[1]).send(path.last)
    end
  end
end

class Hash
  def has_rkey?(search)
    search = Regexp.new(search.to_s) unless search.is_a?(Regexp)
    !!keys.detect{ |key| key =~ search }
  end
end
